@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2010 Wilmer van der Gaast and others                *
+  * Copyright 2002-2013 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* The IRC-based UI (for now the only one)                              */
@@ -59,8 +59,10 @@ typedef enum
 	OPER_HACK_IDENTIFY_NOLOAD = 0x01100,
 	OPER_HACK_IDENTIFY_FORCE  = 0x02100,
 	OPER_HACK_REGISTER = 0x200,
-	OPER_HACK_ACCOUNT_ADD = 0x400,
+	OPER_HACK_ACCOUNT_PASSWORD = 0x400,
 	OPER_HACK_ANY = 0x3700,	/* To check for them all at once. */
+	
+	IRC_UTF8_NICKS = 0x10000, /* Disable ASCII restrictions on buddy nicks. */
 } irc_status_t;
 
 struct irc_user;
@@ -206,11 +208,13 @@ typedef struct irc_channel_user
 
 typedef enum
 {
-	IRC_CC_TYPE_DEFAULT,
-	IRC_CC_TYPE_REST,
-	IRC_CC_TYPE_GROUP,
-	IRC_CC_TYPE_ACCOUNT,
-	IRC_CC_TYPE_PROTOCOL,
+	IRC_CC_TYPE_DEFAULT  = 0x00001,
+	IRC_CC_TYPE_REST     = 0x00002, /* Still not implemented. */
+	IRC_CC_TYPE_GROUP    = 0x00004,
+	IRC_CC_TYPE_ACCOUNT  = 0x00008,
+	IRC_CC_TYPE_PROTOCOL = 0x00010,
+	IRC_CC_TYPE_MASK     = 0x000ff,
+	IRC_CC_TYPE_INVERT   = 0x00100,
 } irc_control_channel_type_t;
 
 struct irc_control_channel
