@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2010 Wilmer van der Gaast and others                *
+  * Copyright 2002-2012 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /* Some glue to put the IRC and the IM stuff together.                  */
@@ -70,7 +70,17 @@ static gboolean bee_irc_user_new( bee_t *bee, bee_user_t *bu )
 		if( bu->ic->acc->server )
 			iu->host = g_strdup( bu->ic->acc->server );
 		else
-			iu->host = g_strdup( bu->ic->acc->prpl->name );
+		{
+			char *s;
+			for( s = bu->ic->acc->tag; isalnum( *s ); s ++ );
+			/* Only use the tag if we got to the end of the string.
+			   (So allow alphanumerics only. Hopefully not too
+			   restrictive.) */
+			if( *s )
+				iu->host = g_strdup( bu->ic->acc->prpl->name );
+			else
+				iu->host = g_strdup( bu->ic->acc->tag );
+		}
 	}
 	
 	while( ( s = strchr( iu->user, ' ' ) ) )
@@ -615,7 +625,7 @@ static gboolean bee_irc_chat_msg( bee_t *bee, struct groupchat *c, bee_user_t *b
 	irc_t *irc = bee->ui_data;
 	irc_user_t *iu = bu->ui_data;
 	irc_channel_t *ic = c->ui_data;
-	char *ts = NULL;
+	char *wrapped, *ts = NULL;
 	
 	if( ic == NULL )
 		return FALSE;
@@ -623,7 +633,8 @@ static gboolean bee_irc_chat_msg( bee_t *bee, struct groupchat *c, bee_user_t *b
 	if( sent_at > 0 && set_getbool( &bee->set, "display_timestamps" ) )
 		ts = irc_format_timestamp( irc, sent_at );
 	
-	irc_send_msg( iu, "PRIVMSG", ic->name, msg, ts );
+	wrapped = word_wrap( msg, 425 );
+	irc_send_msg( iu, "PRIVMSG", ic->name, wrapped, ts );
 	g_free( ts );
 	
 	return TRUE;
@@ -696,7 +707,7 @@ static gboolean bee_irc_chat_name_hint( bee_t *bee, struct groupchat *c, const c
 	stripped[MAX_NICK_LENGTH] = '\0';
 	irc_channel_name_strip( stripped );
 	if( set_getbool( &bee->set, "lcnicks" ) )
-		nick_lc( stripped );
+		nick_lc( irc, stripped );
 	
 	if( stripped[0] == '\0' )
 		return FALSE;
