@@ -4,7 +4,7 @@
 *  Simple module to facilitate twitter functionality.                       *
 *                                                                           *
 *  Copyright 2009-2010 Geert Mulders <g.c.w.m.mulders@gmail.com>            *
-*  Copyright 2010-2012 Wilmer van der Gaast <wilmer@gaast.net>              *
+*  Copyright 2010-2013 Wilmer van der Gaast <wilmer@gaast.net>              *
 *                                                                           *
 *  This library is free software; you can redistribute it and/or            *
 *  modify it under the terms of the GNU Lesser General Public               *
@@ -39,14 +39,6 @@
 #include <ctype.h>
 #include <errno.h>
 
-/* GLib < 2.12.0 doesn't have g_ascii_strtoll(), work around using system strtoll(). */
-/* GLib < 2.12.4 can be buggy: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=488013 */
-#if !GLIB_CHECK_VERSION(2,12,5)
-#include <stdlib.h>
-#include <limits.h>
-#define g_ascii_strtoll strtoll
-#endif
-
 #define TXL_STATUS 1
 #define TXL_USER 2
 #define TXL_ID 3
@@ -171,7 +163,7 @@ char *twitter_parse_error(struct http_request *req)
 	ret = NULL;
 
 	if (req->body_size > 0) {
-		root = json_parse(req->reply_body);
+		root = json_parse(req->reply_body, req->body_size);
 		err = json_o_get(root, "errors");
 		if (err && err->type == json_array && (err = err->u.array.values[0]) &&
 		    err->type == json_object) {
@@ -227,7 +219,7 @@ static json_value *twitter_parse_response(struct im_connection *ic, struct http_
 		td->http_fails = 0;
 	}
 
-	if ((ret = json_parse(req->reply_body)) == NULL) {
+	if ((ret = json_parse(req->reply_body, req->body_size)) == NULL) {
 		imcb_error(ic, "Could not retrieve %s: %s",
 			   path, "XML parse error");
 	}
@@ -270,7 +262,7 @@ static gboolean twitter_xt_get_friends_id_list(json_value *node, struct twitter_
 			continue;
 		
 		txl->list = g_slist_prepend(txl->list,
-			g_strdup_printf("%lld", c->u.array.values[i]->u.integer));
+			g_strdup_printf("%" PRIu64, c->u.array.values[i]->u.integer));
 	}
 	
 	c = json_o_get(node, "next_cursor");
@@ -780,7 +772,7 @@ static void twitter_http_stream(struct http_request *req)
 		c = req->reply_body[len];
 		req->reply_body[len] = '\0';
 		
-		if ((parsed = json_parse(req->reply_body))) {
+		if ((parsed = json_parse(req->reply_body, req->body_size))) {
 			twitter_stream_handle_object(ic, parsed);
 		}
 		json_value_free(parsed);
@@ -907,7 +899,7 @@ static void twitter_get_mentions(struct im_connection *ic, gint64 next_cursor);
 /**
  * Get the timeline with optionally mentions
  */
-void twitter_get_timeline(struct im_connection *ic, gint64 next_cursor)
+gboolean twitter_get_timeline(struct im_connection *ic, gint64 next_cursor)
 {
 	struct twitter_data *td = ic->proto_data;
 	gboolean include_mentions = set_getbool(&ic->acc->set, "fetch_mentions");
@@ -916,6 +908,7 @@ void twitter_get_timeline(struct im_connection *ic, gint64 next_cursor)
 		if (++td->http_fails >= 5) {
 			imcb_error(ic, "Fetch timeout (%d)", td->flags);
 			imc_logout(ic, TRUE);
+			return FALSE;
 		}
 	}
 
@@ -926,6 +919,8 @@ void twitter_get_timeline(struct im_connection *ic, gint64 next_cursor)
 	if (include_mentions) {
 		twitter_get_mentions(ic, next_cursor);
 	}
+	
+	return TRUE;
 }
 
 /**
@@ -1241,10 +1236,12 @@ void twitter_report_spam(struct im_connection *ic, char *screen_name)
  */
 void twitter_favourite_tweet(struct im_connection *ic, guint64 id)
 {
-	char *url;
-	url = g_strdup_printf("%s%llu%s", TWITTER_FAVORITE_CREATE_URL,
-	                      (unsigned long long) id, ".json");
-	twitter_http_f(ic, url, twitter_http_post, ic, 1, NULL, 0,
-	               TWITTER_HTTP_USER_ACK);
-	g_free(url);
+	char *args[2] = {
+		"id",
+		NULL,
+	};
+	args[1] = g_strdup_printf("%llu", (unsigned long long) id);
+	twitter_http_f(ic, TWITTER_FAVORITE_CREATE_URL, twitter_http_post,
+	               ic, 1, args, 2, TWITTER_HTTP_USER_ACK);
+	g_free(args[1]);
 }
