@@ -44,11 +44,11 @@ extern char *SET_INVALID;
 
 typedef enum
 {
-	SET_NOSAVE = 0x0001,
-	SET_NULL_OK = 0x0100,
-	SET_HIDDEN = 0x0200,
-	SET_PASSWORD = 0x0400,
-	SET_HIDDEN_DEFAULT = 0x0800,
+	SET_NOSAVE = 0x0001,   /* Don't save this setting (i.e. stored elsewhere). */
+	SET_NULL_OK = 0x0100,  /* set->value == NULL is allowed. */
+	SET_HIDDEN = 0x0200,   /* Don't show up in setting lists. Mostly for internal storage. */
+	SET_PASSWORD = 0x0400, /* Value shows up in settings list as "********". */
+	SET_HIDDEN_DEFAULT = 0x0800, /* Hide unless changed from default. */
 } set_flags_t;
 
 typedef struct set
